import csv
import re
from glob import glob
from datetime import datetime

from cutils import ProgressBar
from nltk.corpus import stopwords
from vaderSentiment.vaderSentiment import sentiment as vaderSentiment
# pip install vaderSentiment


STOPWORDS = dict(zip(stopwords.words('english'), [''] * len(stopwords.words('english'))))


def get_sent(vs):
    vs.pop('compound')
    v=vs.values()
    k=vs.keys()
    return k[v.index(max(v))]

def get_sentiment(sentence):
    words = re.sub('[^a-z0-9 ]', ' ', sentence.lower())
    words = re.sub(' +', ' ', words)
    words = ' '.join([word for word in words.split(' ') if word not in STOPWORDS])
    return get_sent(vaderSentiment(words))


# '/Users/cchen224/Downloads/expended_thread_aws/*.txt'
def sentiment_threads(dir_in, fp_out):
    files = glob(dir_in)
    bar = ProgressBar(total=len(files))
    for file in files:
        bar.move().log()
        with open(file, 'r') as i:
            csvreader = csv.reader(i, delimiter='\t')
            with open(fp_out, 'a') as o:
                csvwriter = csv.writer(o)
                is_write = True
                for row in csvreader:
                    if is_write:
                        tid = row[0]
                        uid = row[1]
                        sentence = row[3].strip()
                        words = re.sub('[^a-z0-9 ]', ' ', sentence.lower())
                        words = re.sub(' +', ' ', words)
                        words = ' '.join([word for word in words.split(' ') if word not in STOPWORDS])
                        csvwriter.writerow([tid, uid, sentence, get_sent(vaderSentiment(words))])
                    is_write = row == []
    bar.close()
# sentiment('/Users/cchen224/Downloads/expended_thread_aws/*.txt', '/Users/cchen224/Downloads/sentiments.csv')


def sentiment(fp_in, fp_out):
    with open(fp_in, 'r') as i:
        csvreader = csv.reader(i)
        with open(fp_out, 'w') as o:
            csvwriter = csv.writer(o)
            for row in csvreader:
                tid = row[0]
                uid = row[1]
                sentence = row[3].strip()
                words = re.sub('[^a-z0-9 ]', ' ', sentence.lower())
                words = re.sub(' +', ' ', words)
                words = ' '.join([word for word in words.split(' ') if word not in STOPWORDS])

                sentiment = get_sent(vaderSentiment(words))

                row.pop(6)
                row.pop(5)
                row.pop(4)
                row.append(sentiment)
                csvwriter.writerow(row)
# sentiment('/Users/../Volumes/cchen224/tweets_07_ads.csv', '/Users/../Volumes/cchen224/tweets_07_ads_sentiment.csv')
sentiment('/Users/../Volumes/cchen224/tweets_06_ads.csv', '/Users/../Volumes/cchen224/tweets_06_ads_sentiment.csv')


import csv
import random
import re
import sys
import time
from datetime import datetime
from sys import stdout

import requests
from bs4 import BeautifulSoup

from cutils import Gmail
from cutils import ProgressBar


def parse_tweet_card(uid, tid):
    # out = {'n_uid': '', 'n_tid': '', 'has_result': True}
    url = 'https://twitter.com/' + uid + '/status/' + tid
    html = None

    for count in xrange(5):
        try:
            html = requests.request('GET', url).text.encode('ascii', 'ignore')
            break
        except requests.ConnectionError:
            time.sleep(60)
            stdout.write(' Connection error, sleeping 1 min.', str(count + 1) + '\r')
            sys.stdout.flush()
            continue
    if html is None:
        return None, None

    # stdout.write(' ' + tid + '\r')
    # stdout.flush()
    soup = BeautifulSoup(html, 'html.parser')
    ancestors = soup.find('div', {'id': 'ancestors'})
    if ancestors is None:
        original = soup.find('div', class_='permalink-inner permalink-tweet-container')
        original = parse_tweet_cardwrap(original)
        out = []
        try:
            out.extend(parse_threads_sequentialTweets(soup))
            out.extend(parse_threads_loneTweet(soup))
        except AttributeError:
            out = []
    else:
        original = parse_tweet_cardwrap(ancestors)
        original, out = parse_tweet_card(original[2], original[1])

    return original, out



def parse_threads_sequentialTweets(soup):
    out = []
    for thread_soup in soup.find_all('li', class_='ThreadedConversation'):
        threads = dict()

        for tweet_soup in thread_soup.find_all('div', class_=re.compile('^ThreadedConversation-tweet')):
            parsed_tweet = parse_tweet_cardwrap(tweet_soup)
            threads[parsed_tweet[0] + '_' + parsed_tweet[1]] = parsed_tweet

        view_others = thread_soup.find_all('li', class_='ThreadedConversation-viewOther')
        if len(view_others) > 0:
            for view_other in view_others:
                view_other_url = 'https://twitter.com' + view_other.a.attrs['href']
                view_other_html = requests.request('GET', view_other_url).text.encode('ascii', 'ignore')
                view_other_soup = BeautifulSoup(view_other_html, 'html.parser')
                for view_other_tweet in parse_threads_loneTweet(view_other_soup):
                    threads[view_other_tweet[0] + '_' + view_other_tweet[1]] = view_other_tweet
        out.append([threads[key] for key in sorted(threads.keys())])
    return out


def parse_threads_loneTweet(soup):
    return [[parse_tweet_cardwrap(lone)] for lone in soup.find_all('div', class_='ThreadedConversation--loneTweet')]


def parse_tweet_cardwrap(tweet_soup):
    properties = tweet_soup.find('div', class_=re.compile('tweet')).attrs
    text_container = tweet_soup.find('div', class_='js-tweet-text-container')
    footer = tweet_soup.find('div', class_='stream-item-footer')

    timestamp = tweet_soup.find('span', class_=re.compile('js-short-timestamp')).attrs['data-time']
    timestamp = int(timestamp) + 6 * 60 * 60
    timestamp = datetime.fromtimestamp(timestamp).strftime('%Y-%m-%d %H:%M:%S')

    text = re.sub('\n|\t', ' ', text_container.text.strip())
    text = re.sub('(http|pic)', ' \\1', text)
    text = re.sub(' +', ' ', text)

    retweet = footer.find('span', class_='ProfileTweet-action--retweet u-hiddenVisually').text.strip()
    retweet = re.sub('[^0-9]', '', retweet)
    likes = footer.find('span', class_='ProfileTweet-action--favorite u-hiddenVisually').text.strip()
    likes = re.sub('[^0-9]', '', likes)

    # datetime,tid,uscreenname,text,retweet,likes
    return [timestamp,
            properties['data-tweet-id'],
            properties['data-screen-name'],
            text.encode('ascii', 'ignore'),
            retweet, likes]


def writer(output, original, thread):
    with open(output, 'a') as o:
        csvwriter = csv.writer(o, delimiter='\t')
        if thread:
            # for items in out:
            csvwriter.writerow(original)
            for item in thread:
                csvwriter.writerow(item)
            o.write('\n')
        else:
            csvwriter.writerow(original)
            o.write('\n')


if __name__ == '__main__':
    argvs = sys.argv[1:]
    input = argvs[0]
    output = argvs[1]

    out = dict()
    with open(input, 'r') as i:
        csvreader = csv.reader(i, delimiter=',', quotechar='"')
        keys = [row[10] + '_' + row[1] for row in csvreader]

    counter = 0
    bar = ProgressBar(total=len(keys))
    print input, 'is being crawled.'
    for key in keys:
        bar.move().log()
        try:
            counter += 1
            n = len(out)
            out[key] = 1

            if counter % 100000 == 0:
                time.sleep(random.uniform(3,8))
            if len(out) > n:
                n = len(out)
                tokens = key.split('_')
                # print tokens
                original, parsed_tweet = parse_tweet_card(tokens[0], tokens[1])

                if original[2] + '_' + original[1] != key:  # make sure no duplicate
                    out[original[2] + '_' + original[1]] = 1
                    if len(out) == n:
                        continue
                if parsed_tweet:  # make sure target brand is crawled
                    for items in parsed_tweet:
                        total_text = original[3] + ' '
                        for item in items:
                            total_text += item[3]
                        if input.split('/')[-1].split('.')[0] in total_text:
                            writer(output, original, items)
                else:
                    writer(output, original, parsed_tweet)
        except Exception:
            pass

    mail= Gmail() \
        .set_from_to(receiver='wonderfulhoo@gmail.com') \
        .set_subject('AWS Status Brand') \
        .set_credentials('/Users/cchen224/Downloads/') \
        .send_message(input + ' Done!\n') \
        .close()
    bar.close()



import json

from bs4 import BeautifulSoup
from cutils import Gmail
import traceback
from time import sleep
from random import random
from sys import stdout

from webparser_utility import *

HEADERS = {'authority': 'twitter.com',
           'method': 'GET',
           # 'path': '/i/trends?k=&lang=en&pc=true&query=from%3ATomaasNavarrete+since%3A2016-02-05+until%3A2016-02-10&show_context=true&src=module',
           'scheme': 'https',
           'accept': 'application/json, text/javascript, */*; q=0.01',
           'accept-encoding': 'gzip, deflate, sdch',
           'accept-language': 'en-US,en;q=0.8',
           # 'cookie': 'lang=en; guest_id=v1%3A146359191342153342; pid="v3:1463591915583776897984958"; _ga=GA1.2.664361646.1463591916; _twitter_sess=BAh7CSIKZmxhc2hJQzonQWN0aW9uQ29udHJvbGxlcjo6Rmxhc2g6OkZsYXNo%250ASGFzaHsABjoKQHVzZWR7ADoPY3JlYXRlZF9hdGwrCNlH4MRUAToMY3NyZl9p%250AZCIlNGI0MDI2ZDg4NGU1ZjFhYzUxMWU3MTlkNTY1MDNiMTk6B2lkIiU1NjEw%250AZmYwMzZjZGRjMzc3OTRhYjIwMWU4MWQ2ZDJiZA%253D%253D--f8d4b86ec2f08169404dc0049185ac0557375054',
           # 'referer': 'https://twitter.com/search?q=from%3ATomaasNavarrete%20since%3A2016-02-05%20until%3A2016-02-10&src=typd&lang=en',
           'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36',
           'x-requested-with': 'XMLHttpRequest'}


def parse_user_timeline(screen_name, **kwargs):
    adv_search = kwargs.get('adv_search', dict())
    date_since = adv_search.get('date_since', '')
    date_until = adv_search.get('date_until', '')

    cur_page = 0
    session = requests.Session()
    adapter = requests.adapters.HTTPAdapter(max_retries=10)
    session.mount('https://', adapter)

    out = []
    tid_start = ''
    has_next = True
    while has_next:
        cur_page += 1
        stdout.write(str(cur_page) + '\r')
        stdout.flush()
        if adv_search:
            if cur_page == 1:
                url = 'https://twitter.com/search?q=from%3A' + screen_name + \
                      '%20since%3A' + date_since + \
                      '%20until%3A' + date_until + '&src=typd&lang=en'
                html = session.get(url, headers=HEADERS)._content
            else:
                url = 'https://twitter.com/i/search/timeline?vertical=default&q=from%3A' + screen_name + \
                      '%20since%3A' + date_since + \
                      '%20until%3A' + date_until + \
                      '&src=typd&include_available_features=1&include_entities=1&' + \
                      'max_position=TWEET-' + tid + '-' + tid_start + '-' + \
                      'BD1UO2FFu9QAAAAAAAAETAAAAAcAAAASAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA' + \
                      '&reset_error_state=false'
                for counter in range(5):
                    html_json = json.loads(session.get(url, headers=HEADERS)._content)
                    if html_json.get('errors', ''):
                        sleep(random() * 10)
                    else:
                        break
                html = html_json['items_html'].strip()
                has_next = html_json['has_more_items']
                    # Gmail().set_subject('AWS Twitter.timeline Error').set_from_to()\
                    #     .send_message(screen_name + '\n' + str(html_json) + '\n' + has_next.__str__() + '\n\n\n' + traceback.format_exc()).close()
        else:
            if cur_page == 1:
                url = 'https://twitter.com/' + re.sub('\@', '', screen_name)
                html = session.get(url)._content
            else:
                url = 'https://twitter.com/i/profiles/show/' \
                      + re.sub('\@', '', screen_name) \
                      + '/timeline/tweets?include_available_features=1&include_entities=1&max_position=' \
                      + tid \
                      + '&reset_error_state=false'
                html_json = json.loads(session.get(url, headers=HEADERS)._content)
                html = html_json['items_html'].strip()
                has_next = html_json['has_more_items']
        soup = BeautifulSoup(html, 'html.parser')

        tweet_cardwraps = soup.find_all('li', class_='js-stream-item stream-item stream-item expanding-stream-item ')
        if len(tweet_cardwraps) == 0:
            break

        for tweet_cardwrap in tweet_cardwraps:
            uid, screen_name, tid = parse_properties(tweet_cardwrap)
            # if not uid:
            #     return out
            if not tid_start:
                tid_start = tid

            [timestamp] = parse_header(tweet_cardwrap)
            text, hashtags, cashtags, mentions, emojis = \
                parse_text(tweet_cardwrap.find('div', class_='js-tweet-text-container'))
            retweets, likes = parse_footer(tweet_cardwrap.find('div', class_='stream-item-footer'))
            out.append([uid, screen_name, tid, timestamp, text, hashtags, cashtags, mentions, emojis, retweets, likes])

    return out


# with open('/Users/cchen224/Downloads/testadvsearch.csv', 'w') as i:
#     csvwriter = csv.writer(i)
#     for out in parse_user_timeline('GingerRepublic', adv_search={'date_since':'2016-02-05', 'date_until': '2016-02-09'}):
#         csvwriter.writerow(out)
    #
    # with open('/Users/cchen224/Downloads/testadvsearch', 'w') as i:
    #     # i.write(html.encode('utf-8'))
    #     i.write(json.loads(session.get(url, headers=HEADERS)._content)['module_html'])



def is_page_exists(soup):
    try:
        content = soup.find('div', class_='body-content').h1.text.strip()
        if content == 'Sorry, that page doesnt exist!':
            return False
    except AttributeError:
        return True


def is_not_content_protected(soup):
    try:
        content = soup.find('div', class_='message-inside').span.text.strip()
        if content == 'Sorry, you are not authorized to see this status.':
            return False
        else: return True
    except AttributeError:
        return True


def is_not_account_suspended(soup):
    try:
        content = soup.find('div', class_='flex-module error-page clearfix').h1.text.strip()
        if content == 'Account suspended':
            return False
    except AttributeError:
        return True

def is_not_tweet_withheld(soup):
    try:
        content = soup.find('div', class_='tweet-user-withheld').text.strip()
        if content == 'Tweet withheld':
            return False
    except AttributeError:
        return True

def has_result(soup):
    try:
        content = soup.find('p', class_='empty-text').text.strip()
        if content == 'No results.':
            return False
    except AttributeError:
        return True


from twitter_accounts import accounts
from twitter_followers import get_followers_ids

NFL0 = ['BuffaloBills', 'MiamiDolphins', 'RealPatriots', 'NYJets',
        '1WinningDrive', 'Bengals', 'OfficialBrowns', 'Steelers',
        'HoustonTexans', 'NFLColts', 'JaguarsInsider']
NFL1 = ['TennesseeTitans',
        'Denver_Broncos', 'KCChiefs', 'Raiders', 'Chargers',
        'DallasCowboys', 'Giants', 'Eagles', 'Redskins']
NFL2 = ['ChicagoBearscom', 'DetroitLionsNFL', 'Packers', 'VikingsFootball',
        'Atlanta_Falcons', 'Panthers', 'Official_Saints', 'TBBuccaneers',
        'AZCardinals', 'STLouisRams', '49ers', 'Seahawks']

for team in NFL1:
        get_followers_ids(team, accounts[2])





import csv
from datetime import datetime

def filter(fp_in, fp_out):
    with open(fp_in, 'r') as i:
        csvreader = csv.reader(i)
        with open(fp_out, 'w') as o:
            csvwriter = csv.writer(o)
            for row in csvreader:
                if int(row[4]) > 20:
                    csvwriter.writerow(row)


def merge_post(fp_in, fp_out, user_info, **kwargs):
    # user_id, create_at, location_id, latitude, longitude,
    try:
        user_id = kwargs['user_id']
    except KeyError:
        user_id = 0
    try:
        create_at = kwargs['create_at']
    except KeyError:
        create_at = 1
    try:
        location_id = kwargs['location_id']
    except KeyError:
        location_id = 2
    try:
        latitude = kwargs['latitude']
    except KeyError:
        latitude = 3
    try:
        longitude = kwargs['longitude']
    except KeyError:
        longitude = 4

    info = dict()
    with open(user_info, 'r') as i:
        csvreader = csv.reader(i)
        for row in csvreader:
            info[row[0]] = row[2:5]

    post = dict()
    with open(fp_in, 'r') as i:
        csvreader = csv.reader(i)
        for row in csvreader:
            if row[user_id] in info:
                if row[user_id] not in post:
                    post[row[user_id]] = [row[create_at] + '@' +
                                          row[location_id] +
                                          '->' + row[latitude] + ',' + row[longitude]]
                else:
                    post[row[user_id]].append(row[create_at] + '@' +
                                               row[location_id] +
                                               '->' + row[latitude] + ',' + row[longitude])

    with open(fp_out, 'w') as o:
        csvwriter = csv.writer(o)
        for key in info.keys():
            row = [key]
            row.extend(info[key])
            row.append('|'.join(post[key]))
            csvwriter.writerow(row)
# merge_post('/Users/cchen224/Downloads/instagram_stats/locations.csv',
#            '/Users/cchen224/Downloads/instagram_stats/merge_data.csv',
#            '/Users/cchen224/Downloads/instagram_stats/instagram_user_info.csv')


def split_active(fp_in):
    with open(fp_in, 'r') as i:
        csvreader = csv.reader(i)
        for row in csvreader:
            if int(row[3]) > 1200:
                with open(fp_in.split('.')[0] + '_active.csv', 'a') as o:
                    csv.writer(o).writerow(row)
            else:
                with open(fp_in.split('.')[0] + '_regular.csv', 'a') as o:
                    csv.writer(o).writerow(row)
# split_active('/Users/cchen224/Downloads/instagram_stats/merge_data.csv')


def geo_stats(fp_in):
    posts = []
    with open(fp_in, 'r') as i:
        csvreader = csv.reader(i)
        [posts.extend(row[4].split('|')) for row in csvreader]
    geo = [post for post in posts if not post.endswith(',')]
    return len(posts), len(geo), float(len(geo)) / len(posts)
# geo_stats('/Users/cchen224/Downloads/instagram_stats/merge_data.csv')
# geo_stats('/Users/cchen224/Downloads/instagram_stats/merge_data_active.csv')
# geo_stats('/Users/cchen224/Downloads/instagram_stats/merge_data_regular.csv')

from cutils import ProgressBar
def geo_stats_over_time(fp_in, form):
    bar = ProgressBar(total=sum(1 for _ in open(fp_in)))
    out = dict()
    out_g = dict()
    with open(fp_in, 'r') as i:
        csvreader = csv.reader(i)
        for row in csvreader:
            bar.skip_move(p_skip=0.95)
            for post in row[4].split('|'):
                t = datetime.strptime(post.split('@')[0], '%Y-%m-%d %H:%M:%S').strftime(form)
                if t not in out:
                    out[t] = 1
                    out_g[t] = 0
                else:
                    out[t] += 1
                if not post.endswith(','):
                    out_g[t] += 1
        bar.close()
    return out, out_g

def get_stats(overall, geo_coded, outputFile):
    out = dict()
    for key in overall.keys():
        out[key] = [overall[key], geo_coded[key]]
    with open(outputFile, 'a') as o:
        csvwriter = csv.writer(o)
        for key in out.keys():
            writerow = [key]
            writerow.extend(out[key])
            csvwriter.writerow(writerow)
# a,b = geo_stats_over_time('/Users/cchen224/Downloads/instagram_stats/merge_data.csv', '%Y')
# get_stats(a, b, '/Users/cchen224/Downloads/instagram_stats/y_overall_stats.csv')
# a,b = geo_stats_over_time('/Users/cchen224/Downloads/instagram_stats/merge_data.csv', '%m')
# get_stats(a, b, '/Users/cchen224/Downloads/instagram_stats/m_overall_stats.csv')
# a,b = geo_stats_over_time('/Users/cchen224/Downloads/instagram_stats/merge_data.csv', '%w')
# get_stats(a, b, '/Users/cchen224/Downloads/instagram_stats/w_overall_stats.csv')
# a,b = geo_stats_over_time('/Users/cchen224/Downloads/instagram_stats/merge_data.csv', '%j')
# get_stats(a, b, '/Users/cchen224/Downloads/instagram_stats/j_overall_stats.csv')
# a,b = geo_stats_over_time('/Users/cchen224/Downloads/instagram_stats/merge_data_active.csv', '%j')
# get_stats(a, b, '/Users/cchen224/Downloads/instagram_stats/j_active_stats.csv')
# a,b = geo_stats_over_time('/Users/cchen224/Downloads/instagram_stats/merge_data_active.csv', '%m')
# get_stats(a, b, '/Users/cchen224/Downloads/instagram_stats/m_active_stats.csv')
# a,b = geo_stats_over_time('/Users/cchen224/Downloads/instagram_stats/merge_data_active.csv', '%w')
# get_stats(a, b, '/Users/cchen224/Downloads/instagram_stats/w_active_stats.csv')
# a,b = geo_stats_over_time('/Users/cchen224/Downloads/instagram_stats/merge_data_active.csv', '%Y')
# get_stats(a, b, '/Users/cchen224/Downloads/instagram_stats/y_active_stats.csv')
# a,b = geo_stats_over_time('/Users/cchen224/Downloads/instagram_stats/merge_data_regular.csv', '%w')
# get_stats(a, b, '/Users/cchen224/Downloads/instagram_stats/w_regular_stats.csv')
# a,b = geo_stats_over_time('/Users/cchen224/Downloads/instagram_stats/merge_data_regular.csv', '%j')
# get_stats(a, b, '/Users/cchen224/Downloads/instagram_stats/j_regular_stats.csv')
# a,b = geo_stats_over_time('/Users/cchen224/Downloads/instagram_stats/merge_data_regular.csv', '%m')
# get_stats(a, b, '/Users/cchen224/Downloads/instagram_stats/m_regular_stats.csv')
# a,b = geo_stats_over_time('/Users/cchen224/Downloads/instagram_stats/merge_data_regular.csv', '%Y')
# get_stats(a, b, '/Users/cchen224/Downloads/instagram_stats/y_regular_stats.csv')

# a,b = geo_stats_over_time('/Users/cchen224/Downloads/instagram_stats/merge_data_regular.csv', '%H')
# get_stats(a, b, '/Users/cchen224/Downloads/instagram_stats/h_regular_stats.csv')
# a,b = geo_stats_over_time('/Users/cchen224/Downloads/instagram_stats/merge_data.csv', '%H')
# get_stats(a, b, '/Users/cchen224/Downloads/instagram_stats/h_overall_stats.csv')
# a,b = geo_stats_over_time('/Users/cchen224/Downloads/instagram_stats/merge_data_active.csv', '%H')
# get_stats(a, b, '/Users/cchen224/Downloads/instagram_stats/h_active_stats.csv')


# locations = []
# with open('/Users/cchen224/Downloads/instagram_stats/merge_data.csv', 'r') as i:
#     csvreader = csv.reader(i)
#     for row in csvreader:
#         locations.extend([post.split('@')[1].split('->')[1] for post in row[4].split('|')])
#
# out = dict()
# for location in locations:
#     if location != ',':
#         tokens = location.split(',')
#         latlon = tokens[0] + ',' + tokens[1]
#     if latlon not in out:
#         out[latlon] = ''
# with open('/Users/cchen224/Downloads/instagram_stats/locations.txt', 'w') as o:
#     [o.write(latlon + '\n') for latlon in out.keys()]
#
#

def get_nposts_per_user(fp_in, fp_out):
    with open(fp_in, 'r') as i:
        csvreader = csv.reader(i)
        with open(fp_out, 'w') as o:
            csvwriter = csv.writer(o)
            for row in csvreader:
                posts = row[4].split('|')
                geo = [ post for post in posts if not post.endswith('->,') ]
                csvwriter.writerow([row[0], len(posts), len(geo)])

# get_nposts_per_user('/Users/cchen224/Downloads/instagram_stats/merge_data_active.csv',
#                     '/Users/cchen224/Downloads/instagram_stats/posts_per_user_active.csv')
# get_nposts_per_user('/Users/cchen224/Downloads/instagram_stats/merge_data_regular.csv',
#                     '/Users/cchen224/Downloads/instagram_stats/posts_per_user_regular.csv')
# get_nposts_per_user('/Users/cchen224/Downloads/instagram_stats/merge_data.csv',
#                     '/Users/cchen224/Downloads/instagram_stats/posts_per_user.csv')




import time

import tweepy


def get_followers_ids(user_id, account):

    auth = tweepy.OAuthHandler(account['consumer_key'], account['consumer_secret'])
    auth.set_access_token(account['access_key'], account['access_secret'])
    api = tweepy.API(auth)

    # f = open(user_id + '_followings.txt', 'w+')
    current_page = 1
    user_timeline = tweepy.Cursor(api.user_timeline, user_id=user_id, since_id=696163960001679361, max_id=None, lang='en').pages()
    # print 'Starting collect ' + screen_name
    while True:
        try:
            print '\tGetting page ' + str(current_page)
            ids = user_timeline.next()
            # for i in ids:
                # f.write(str(i) + '\n')
            current_page += 1
            time.sleep(60)
        except tweepy.TweepError:
            print '\tWaiting on rate limit.'
            time.sleep(60)
        except StopIteration:
            # print screen_name + ' Done!\n\n\n'
            break
    f.close()


import json
f = open('/Users/cchen224/Downloads/tweets_07.json')
file = json.load(f)



user_id=2634592777


import time

import tweepy


def get_followers_ids(screen_name, account):

    auth = tweepy.OAuthHandler(account['consumer_key'], account['consumer_secret'])
    auth.set_access_token(account['access_key'], account['access_secret'])
    api = tweepy.API(auth)

    f = open(screen_name + '_followings.txt', 'w+')
    current_page = 1
    followings = tweepy.Cursor(api.friends_ids, screen_name=screen_name).pages()
    print 'Starting collect ' + screen_name
    while True:
        try:
            print '\tGetting page ' + str(current_page)
            ids = followings.next()
            for i in ids:
                f.write(str(i) + '\n')
            current_page += 1
            time.sleep(60)
        except tweepy.TweepError:
            print '\tWaiting on rate limit.'
            time.sleep(60)
        except StopIteration:
            print screen_name + ' Done!\n\n\n'
            break
    f.close()



import csv
import traceback
from random import random
from time import sleep

from cutils import Gmail
from cutils import ProgressBar
from sys import stdout
from webparser_tweet import parse_tweet
from webparser_user_info import parse_user_info
from webparser_user_timeline import parse_user_timeline


class TwitterCrawler:

    def __init__(self, job):
        self._job = job.lower()
        self._p_sleep = 0.999
        self._t_sleep = 10
        self._input_fp = ''
        self._output_fp = ''
        self._hspace = 1
        self._date_range = dict()

        if 'info' in self._job.lower():
            self._extension = '.info'
        elif 'timeline' in self._job.lower():
            self._extension = '.timeline'
        elif 'threads' in self._job.lower():
            self._extension = '.threads'
        elif 'tweet' in self._job.lower():
            self._extension = '.tweet'

    def set_input(self, input, **kwargs):
        if isinstance(input, str):
            delimiter = kwargs.get('delimiter', ',')
            index = kwargs.get('index', 0)
            start_value = kwargs.get('start_value', '')
            self._input_fp = input
            with open(input, 'r') as i:
                csvreader = csv.reader(i, delimiter=delimiter)
                if isinstance(index, int):
                    self._input = [row[index] for row in csvreader]
                if isinstance(index, dict):
                    self._input = [row[index.get('tid', 1)] + '@' + row[index.get('uid', index.get('screen_name', 0))] for row in csvreader]
            self._input_total = len(self._input)
            try:
                self._start_index = self._input.index(start_value) if start_value != '' else 0
            except:
                self._start_index = 0
            self._input = self._input[(self._start_index + 1) :]
        if isinstance(input, list):
            self._input = input
            self._input_fp = ''
        return self

    def set_output(self, output_fp, output_delimiter=','):
        self._output_fp = output_fp
        self._output_delimiter = output_delimiter
        return self

    def set_notifier(self, notifier_type, **kwargs):
        if 'gmail' in notifier_type.lower():
            credential_fp = kwargs.get('credential', './')
            self._notifier = Gmail() \
                .set_from_to() \
                .set_credentials(credential_fp) \
                .set_subject('AWS Twitter' + self._extension) \
                .send_message(self._input_fp + '\n\n\n')
            self._notifier_error = Gmail() \
                .set_from_to() \
                .set_credentials(credential_fp) \
                .set_subject('AWS Twitter' + self._extension + ' Error!') \
                .send_message(self._input_fp + '\n\n\n')
        return self

    def set_date_range(self, date_since, date_until):
        self._date_range = {'date_since': date_since, 'date_until': date_until}
        return self

    def log(self, message, is_close=False):
        if isinstance(self._notifier, Gmail):
            self._notifier.send_message(message)
        if is_close:
            self._notifier.close()
            self._notifier.send_message(self._input_fp + '\n\n\n')

    def set_progress_bar(self, hspace=1):
        self._hspace = int(hspace)
        return self

    def throw(self, message):
        if isinstance(self._notifier_error, Gmail):
            self._notifier_error.send_message(message).close()
            self._notifier_error.send_message(self._input_fp + '\n\n\n')

    def crawl(self):
        bar = ProgressBar(total=self._input_total, start_count=self._start_index, hspace=self._hspace)
        with open(self._output_fp if self._output_fp != '' else self._input_fp + self._extension, 'a') as o:
            csvwriter = csv.writer(o, delimiter = self._output_delimiter)
            print self._input_fp, ':'
            for item in self._input:
                bar.move().log()
                if random() > self._p_sleep:
                    t = random() * self._t_sleep
                    stdout.write('Random sleeping' + str(t) + 'sec\r')
                    stdout.flush()
                    sleep(t)
                parsed_items = self.parse(item)
                self.log(' ' + item + '...')
                try:
                    for parsed_item in parsed_items:
                        if parsed_item:
                            csvwriter.writerow(parsed_item)
                    self.log('\n')
                except:
                    self.throw(self._input_fp + ':user ' + item + '\n' + traceback.format_exc())
                # bar.skip_move(0.8)
            self.log('\n\n' + '\nDone', True)
        bar.close()

    def record(self):
        self.log('KeyboardInterrupt!\n', True)


    def parse(self, item):
        if self._extension == '.info':
            return parse_user_info(item, picture=False)
        if self._extension == '.tweet':
            return parse_tweet(item)
        if self._extension == '.timeline':
            return parse_user_timeline(item, adv_search=self._date_range)


accounts = [
    {'consumer_key': 'YT0tutGIiNJPN1qgeTbeycyZN',
     'consumer_secret': '3kr7n4wcpSKbJMg2HdUYuv7z6u8Jx5hYeaxEeDunEs94FxRpQI',
     'access_key': '954798157-kSCkePC2Nh6c7XNTDpGBrRW602EG2AAGlxBO742z',
     'access_secret': 'LSDcRb7uMlkzDfmwXlzY7puddMfdBfU8te5cWgbb4S7Rv'},

    {'consumer_key': 'jtsyzmLZh8PVeDFpDESNFhbfx',
     'consumer_secret': 'yf8qgkfrgOnqsLpOgIYRRjXNkWZ9RMSIvizW0MyfdcaWF73GVo',
     'access_key': '4175032576-IEdW79v1rKvaOgySkwlv6px4o7UFYshwACzdaKw',
     'access_secret': 'm6Fs8MzOflylebJoC82Zp8PsO7gAYhJ5TB9OYDULRHdih'},

    {'consumer_key': 'b4cbNQ34QnKrWZisexzevVQ7v',
     'consumer_secret': 'gFvkcmVUgt6BMWgx3MgdMgjuXa1ZAWceR00retenOVowJB2PVz',
     'access_key': '3965724493-Uxnvo0YX9XoqAmaD2L25iO4jgrlzldJas5GPMXM',
     'access_secret': 'GoRdhfiXsEuErQ6ktGDrmXm9x22EdootO9trET9MEUQtG'},
]


from random import random
from datetime import datetime

from bs4 import BeautifulSoup
from time import sleep

from cutils import Gmail
from webparser_error import *
from webparser_utility import *


def parse_tweet(tiduid):
    """
    :param tiduid: tid@uid
    :return: uid, screen_name, tid, timestamp, text, hashtags, cashtags, mentions, emojis, retweets, likes
    """
    tokens = tiduid.split('@')
    tid = tokens[0]
    uid = tokens[1]
    url = 'https://twitter.com/' + uid + '/status/' + tid
    html = get_html(url)

    if html is None:
        return False

    soup = BeautifulSoup(html, 'html.parser')
    error_functions = [is_not_account_suspended(soup),
                       is_not_content_protected(soup),
                       is_page_exists(soup),
                       is_not_tweet_withheld(soup)]
    for fnc in error_functions:
        if not fnc:
            return []

    # PermalinkOverlay
    cardwrap = soup.find('div', class_='permalink-inner permalink-tweet-container')

    try:
        properties = cardwrap.find('div', class_=re.compile('^permalink-tweet')).attrs
        tid = properties['data-item-id']
        uid = properties['data-user-id']
        screen_name = properties['data-screen-name']
    except TypeError:
        t = random() * 60
        print tiduid, 'sleeping', t, 'secs.', datetime.now().strftime('%Y/%m/%d %H:%M:%S')
        Gmail().set_from_to().set_credentials().set_subject('AWS Twitter.tweet PropertiesNotFound').send_message(tiduid + '\n\n' + html.__str__()).close()
        sleep(t)
        return parse_tweet(tiduid)

    out = [uid, screen_name, tid]
    out.extend(parse_header(cardwrap.find('div', class_='permalink-header')))
    out.extend(parse_text(cardwrap.find('div', class_="js-tweet-text-container")))
    out.extend(parse_fixer(cardwrap.find('div', class_='js-tweet-details-fixer tweet-details-fixer')))

    return [out]
